@@ -12,10 +12,9 @@ X, labels_true = make_blobs(n_samples=50, centers=[[1, 1], [-1, -1], [1, -1]], c
 # X, labels_true = make_blobs(n_samples=750, centers=4, n_features=6, cluster_std=0.5)
 # X, labels_true = make_blobs(n_samples=750, centers=6, n_features=10)
 
-
 # %% Silhouette
 # ce = clusteval(method='silhouette', metric='kmeans', savemem=True)
-ce = clusteval(method='silhouette')
+ce = clusteval(method='silhouette', verbose=3)
 results = ce.fit(X)
 ce.plot()
 ce.scatter(X)
@@ -25,7 +24,6 @@ results = ce.dendrogram(max_d=9)
 results = ce.dendrogram(X=X, linkage='single', metric='euclidean')
 results = ce.dendrogram(X=X, linkage='single', metric='euclidean', max_d=0.8)
 results = ce.dendrogram(X=X, linkage='complete', metric='euclidean', max_d=2)
-
 results = ce.dendrogram(figsize=(15,8), show_contracted=True)
 
 results['labx']
@@ -39,15 +37,6 @@ ce.plot()
 ce.scatter(X)
 results = ce.dendrogram()
 
-
-# %% dbscan
-from clusteval import clusteval
-ce = clusteval(method='dbscan')
-results = ce.fit(X)
-ce.plot()
-ce.scatter(X)
-results = ce.dendrogram()
-
 # %% dbindex
 from clusteval import clusteval
 ce = clusteval(method='dbindex')
@@ -63,9 +52,17 @@ results = ce.fit(X)
 ce.plot()
 ce.scatter(X)
 
+# %% dbscan
+from clusteval import clusteval
+ce = clusteval(cluster='dbscan')
+results = ce.fit(X)
+ce.plot()
+ce.scatter(X)
+results = ce.dendrogram()
+
 # %% hdbscan
 from clusteval import clusteval
-ce = clusteval(method='hdbscan')
+ce = clusteval(cluster='hdbscan')
 results = ce.fit(X)
 ce.plot()
 ce.scatter(X)
